#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Defines GameDayClient, the primary class for scraping, parsing, and ingesting MLB GameDay data.
"""
from __future__ import print_function, division, absolute_import

import logging
from datetime import timedelta
from dateutil import parser

from sqlalchemy import func
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm import sessionmaker

import pygameday.util as util
import pygameday.parse as parse
import pygameday.scrape as scrape
from pygameday.models import Game
from pygameday.models import Player
from pygameday.models import AtBat
from pygameday.models import Pitch
from pygameday.models import HitInPlay
from pygameday.models import create_db_tables
from pygameday.models import db_connect

logger = logging.getLogger("pygameday")


class GameDayClient(object):
    """Class for ingesting GameDay data into a database

    Database parameters are defined in config.py.
    """
    def __init__(self, database_uri, ingest_spring_training=False, log_level="INFO"):
        """Constructor

        Initializes database connection and session
        Creates database tables if they do not already exist
        Sets up logging

        Parameters
        ----------
        database_uri : str
            The URI for the database.
            Examples: "sqlite:///gameday.db"  (sqlite)
                      "postgresql+psycopg2://user:passwd@localhost/gameday"  (Postgres via psycopg2)

            See SQLAlchemy's documentation for valid database URIs.

        ingest_spring_training : bool
            Whether to ingest spring training games. [Default: False]

        log_level : str
            The logging level. Must be one of NOTSET, DEBUG, INFO, WARN, ERROR, CRITICAL [Default: "INFO"]
        """
        util.set_logging_level(log_level)

        engine = db_connect(database_uri)
        create_db_tables(engine)
        self.Session = sessionmaker(bind=engine)
        logger.info("Initialized GameDayClient using '{}'".format(database_uri))

        self.database_uri = database_uri
        self.ingest_spring_training = ingest_spring_training
        self.player_ids = set()  # Player IDs that have already been inserted into the database
        self.gameday_ids = set()  # Game IDs that have already been inserted into the database

        self.update_inserted_data()  # Update the set of players and games that are already inserted

    def db_stats(self):
        """Prints information about the current database contents
        """
        session = self.Session()
        game_count = session.query(func.count(Game.game_id)).scalar()
        player_count = session.query(func.count(Player.player_id)).scalar()
        atbat_count = session.query(func.count(AtBat.at_bat_id)).scalar()
        pitch_count = session.query(func.count(Pitch.pitch_id)).scalar()
        hip_count = session.query(func.count(HitInPlay.hip_id)).scalar()
        session.close()

        print("")
        print("======================")
        print("{:^22}".format("DATABASE CONTENTS"))
        print("----------------------")
        print("   TABLE    |  COUNT  ")
        print("------------ ---------")
        print("{: <12} {: >8}".format("Games", game_count))
        print("{: <12} {: >8}".format("At Bats", atbat_count))
        print("{: <12} {: >8}".format("Hits in Play", hip_count))
        print("{: <12} {: >8}".format("Pitches", pitch_count))
        print("{: <12} {: >8}".format("Players", player_count))
        print("======================")
        print("")

    def update_inserted_data(self):
        """Updates the set of player IDs and Game GameDay IDs that already exist in the database

        Keeping track of games and players already inserted saves us from trying to insert objects that are already
        there.  It also helps with limiting the number of times we have to query the database.
        """
        session = self.Session()

        self.gameday_ids = {gid[0] for gid in session.query(Game.gameday_id)}
        self.player_ids = {pid[0] for pid in session.query(Player.player_id)}

        logger.debug("There are currently {} games and {} players in the database".format(
                len(self.gameday_ids), len(self.player_ids)))

    def process_date_range(self, start_date, end_date):
        """Ingests GameDay data within a range of specified dates

        All dates within the begin date and end date are processed.

        Parameters
        ----------
        start_date: datetime.date object
            The first date to process.
            Can also be a string, in which case the function will parse it into a datetime object.

        end_date: datetime.date object
            The final date to process.
            Can also be a string, in which case the function will parse it into a datetime object.
        """
        start_date = util.validate_date(start_date)  # Ensure dates are datetime objects
        end_date = util.validate_date(end_date)

        if end_date < start_date:
            tmp = end_date
            end_date = start_date
            start_date = tmp

        date_range = (end_date - start_date).days + 1  # Add 1 so the range is inclusive of begin_date and end_date

        msg = "Ingesting GameDay data within date range {} to {}".format(start_date.date(), end_date.date())
        logger.info(msg)

        for day in range(date_range):
            date = start_date + timedelta(day)
            self.process_date(date)

    def process_date(self, date):
        """Ingests one day of GameDay data

        Parameters
        ----------
        date : datetime.date object
            The date to process
            Can also be a string, in which case the function will parse it into a datetime object.
        """
        date = util.validate_date(date)  # Ensure date is a datetime object
        epg_page = scrape.fetch_epg(date.year, date.month, date.day)

        if epg_page is None:
            msg = "No games found on {}".format(date.date())
            logger.warn(msg)

        else:
            game_xml_nodes = parse.parse_epg(epg_page)
            msg = "Processing {} games on {}".format(len(game_xml_nodes), date.date())
            logger.info(msg)

            for game in game_xml_nodes:
                self.process_game(game)

    def process_game(self, game_xml_node):
        """Ingests a single game's GameDay data

        Parameters
        ----------
        game_xml_node : lxml node
            The XML node of the game to process
        """
        session = self.Session()

        game_dir = game_xml_node.get("game_data_directory")
        gameday_id = game_xml_node.get("id")

        if gameday_id in self.gameday_ids:
            # The game has been processed and should already be in the database
            logger.warn("Skipping game: {}. It's already in the DB.".format(gameday_id))
            return

        # Parse the game
        db_game = parse.parse_game(game_xml_node)

        # If no data comes back, the game probably wasn't a Final. Abort.
        if db_game is None:
            msg = ("Skipping game: {}. It contained no data, "
                   "probably because its status isn't Final").format(gameday_id)
            logger.warn(msg)
            return

        # If the game is a spring training game, skip it if ingest_spring_training is False
        if not self.ingest_spring_training and (db_game.game_type == "S" or db_game.game_type == "E"):
            msg = "Skipping game: {}. It's a spring training or exhibition game.".format(gameday_id)
            logger.warn(msg)
            return

        msg = "Processing game ID {}".format(gameday_id)
        logger.info(msg)

        # --------------------------------------------------------------------------------------------------------------
        # Fetch game data
        #
        hit_chart_page = scrape.fetch_hit_chart(game_dir)
        players_page = scrape.fetch_players(game_dir)
        inning_all_page = scrape.fetch_inning_all(game_dir)

        # do some error checking
        if hit_chart_page is None:
            msg = "Error fetching hit chart page for game {}".format(gameday_id)
            logger.error(msg)
        if players_page is None:
            msg = "Error fetching players page for game {}".format(gameday_id)
            logger.error(msg)
        if inning_all_page is None:
            msg = "Error fetching inning events page for game {}".format(gameday_id)
            logger.error(msg)

        # --------------------------------------------------------------------------------------------------------------
        # Parse AtBats (including Pitches), HitsInPlay, Players
        #
        db_at_bats = parse.parse_inning_all(inning_all_page)  # Appends Pitches to AtBats
        db_hips = parse.parse_hit_chart(hit_chart_page)
        db_players = parse.parse_players(players_page)

        # --------------------------------------------------------------------------------------------------------------
        # Append the AtBats to the Game. Note that Pitches are appended to AtBats
        # when the AtBats are parsed, so we don't have to do anything with Pitches.
        #
        for db_ab in db_at_bats:
            db_game.at_bats.append(db_ab)

        # --------------------------------------------------------------------------------------------------------------
        # Append the hits in play to the Game
        #
        for db_h in db_hips:
            db_game.hits_in_play.append(db_h)

        # --------------------------------------------------------------------------------------------------------------
        # Add the players using the database session and commit
        # This has to be done one at a time (instead of using session.add_all)
        # because add_all will fail if ANY of the players in the list are
        # duplicated, which could lead to some players being excluded from
        # the database.
        #
        for player in db_players:

            if int(player.player_id) in self.player_ids:
                # The player has been processed and should already be in the database
                logger.debug("Skipping player: {} because it has already been processed.".format(player.player_id))

            else:
                # We haven't inserted this player yet
                error_occurred = False

                try:
                    session.add(player)
                    session.commit()

                except IntegrityError:
                    # If an IntegrityError occurs, it's probably because the data
                    # has already been inserted.
                    session.rollback()
                    msg = ("IntegrityError when inserting player: {}, "
                           "probably because it's already in the database".format(str(player)))
                    logger.error(msg)
                    error_occurred = True

                except Exception as ex:
                    # Just log other exceptions for now, and continue
                    session.rollback()
                    logger.error(str(ex))
                    error_occurred = True

                if not error_occurred:
                    self.player_ids.add(player.player_id)

        # --------------------------------------------------------------------------------------------------------------
        # Add the game data
        #
        if db_game.gameday_id in self.gameday_ids:
            # The game has been processed and should already be in the database
            logger.info("Skipping game: {} because it has already been ingested.".format(db_game.gameday_id))

        else:
            # We haven't inserted this game yet
            error_occurred = False

            try:
                session.add(db_game)
                session.commit()

            except IntegrityError:
                # If an IntegrityError occurs, it's probably because the data
                # has already been inserted.
                session.rollback()
                msg = ("IntegrityError when inserting game: {}, "
                       "probably because it's already in the database".format(db_game.gameday_id))
                logger.error(msg)
                error_occurred = True

            except Exception as ex:
                # Just log other exceptions for now, and continue
                session.rollback()
                logger.error(str(ex))
                error_occurred = True

            if not error_occurred:
                self.gameday_ids.add(db_game.gameday_id)

        # We are done
        session.close()


# ----------------------------------------------------------------------------------------------------------------------
# Unit tests
#
def test_single_day():
    import config
    date_string = "2015-06-07"
    date = parser.parse(date_string)

    ingester = GameDayClient(config.DATABASE_URI)
    ingester.process_date(date)


def test_date_range():
    start_date_string = "2015-06-07"
    end_date_string = "2015-06-07"

    start_date = parser.parse(start_date_string)
    end_date = parser.parse(end_date_string)

    import config
    ingester = GameDayClient(config.DATABASE_URI)
    ingester.process_date_range(start_date, end_date)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Provides functionality for scraping MLB GameDay data from the GameDay website
"""
from __future__ import print_function, division, absolute_import
import requests
import logging

from pygameday.constants import GD_SERVER
from pygameday.constants import GD_BASE_PATH

logger = logging.getLogger("pygameday")


def get_url(url):
    """Fetches a URL, returning the page content

    Parameters
    ----------
    url : str
        The URL to get

    Returns
    -------
    requests response
        The requests page corresonding to the URL
    """
    logger.debug('Fetching URL: {}'.format(url))
    page = requests.get(url)

    if page.status_code != 200:
        logger.error('Error fetching {}'.format(url))
        return None

    return page


def fetch_epg(year, month, day):
    """Fetch epg.xml (possibly stands for "event page"?) for a given day

    Parameters
    ----------
    year : int
        The year that the events took place
    month : int
        The month that the events took place
    day : int
        The day that the events took place

    Returns
    -------
    page : text
        XML-formatted data containing metadata on baseball games that took
        place on the given day.

    """
    url = "{}{}/year_{:d}/month_{:02d}/day_{:02d}/epg.xml".format(GD_SERVER, GD_BASE_PATH, year, month, day)
    return get_url(url)


def fetch_inning_all(game_directory):
    """Fetch the inning_all.xml file for a given game

    The inning_all.xml file for a game contains the full set of game events.

    Parameters
    ----------
    game_directory : str
        The relative path to the game directory

    Returns
    -------
    page : text
        XML-formatted data containing game event data.

    """
    url = GD_SERVER + game_directory + '/inning/inning_all.xml'
    return get_url(url)


def fetch_hit_chart(game_directory):
    """Fetch inning_hit.xml for a given game

    The inning_hit.xml file contains ball-in-play data.

    Parameters
    ----------
    game_directory : str
        The relative path to the game directory

    Returns
    -------
    page : text
        XML-formatted data containing ball-in-play data.

    """
    url = GD_SERVER + game_directory + '/inning/inning_hit.xml'
    return get_url(url)


def fetch_players(game_directory):
    """Fetch players.xml for a given game

    The players.xml file contains player data for the game.

    Parameters
    ----------
    game_directory : str
        The relative path to the game directory

    Returns
    -------
    page : text
        XML-formatted data containing player data.

    """
    url = GD_SERVER + game_directory + '/players.xml'
    return get_url(url)


def save_page(page, filename):
    """Save page content to disk

    Parameters
    ----------
    page : requests get result

    filename : str
        The path to the file that will store the page contents

    """
    with open(filename, 'w') as f:
        f.write(page.text)
        f.close()


def test():
    year = 2014
    month = 4
    day = 4

    print("Fetching epg page")
    epg_page = fetch_epg(year, month, day)
    save_page(epg_page, 'epg.xml')

    game_directory = '/components/game/mlb/year_2014/month_04/day_04/gid_2014_04_04_anamlb_houmlb_1'

    print("Fetching player page")
    players_page = fetch_players(game_directory)
    save_page(players_page, 'players.xml')

    print("Fetching hit chart page")
    hit_chart_page = fetch_hit_chart(game_directory)
    save_page(hit_chart_page, 'inning_hit.xml')

    print("Fetching inning event page")
    inning_event_page = fetch_inning_all(game_directory)
    save_page(inning_event_page, 'inning_all.xml')

#    parser = etree.XMLParser(recover=True, encoding='UTF-8')
#    root = etree.parse(url)
#    root = etree.fromstring(page.content)
#
#    pretty_string = etree.tostring(root, pretty_print=False)
#    f = open('sample_epg.xml', 'w')
#    f.write(pretty_string)
#    f.close()


if __name__ == "__main__":
    test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

import os
import sys
import logging
from datetime import datetime
from dateutil import parser

from pygameday import constants

logger = logging.getLogger("pygameday")

def configure_logging(log_name, log_to_file=False):

    logger = logging.getLogger(log_name)
    log_level = logging.DEBUG
    logger.setLevel(log_level)
    logger.propagate = 0

    # Set handlers
    # Keep track of handlers by name so we don't initialize handlers multiple times
    handler_names = [h.get_name() for h in logger.handlers]

    # Set up a handler to log to file
    if log_to_file and "FileHandler" not in handler_names:
        # First create the log folder if it doesn't exist
        if not os.path.exists(constants.LOG_FOLDER):
            os.mkdir(constants.LOG_FOLDER)

        # Set up a handler to log to file
        today = datetime.now()
        log_file_name = "pygameday_{:4d}-{:2d}-{:2d}.log".format(today.year, today.month, today.day)
        fh = logging.FileHandler(os.path.join(constants.LOG_FOLDER, log_file_name))
        fh.set_name("FileHandler")
        fh.setLevel(log_level)
        file_formatter = logging.Formatter(constants.LOG_FORMAT_FILE, constants.LOG_FORMAT_TIME)
        fh.setFormatter(file_formatter)

    # Set up a handler to log to console
    if "StreamHandler" not in handler_names:
        ch = logging.StreamHandler()
        ch.set_name("StreamHandler")
        ch.setLevel(log_level)  # Can set a different console logging level here
        console_formatter = logging.Formatter(constants.LOG_FORMAT_CONSOLE, constants.LOG_FORMAT_TIME)
        ch.setFormatter(console_formatter)
        logger.addHandler(ch)


def set_logging_level(level):
    """Sets the logging level

    Parameters
    ----------
    level : str
        String corresponding to a valid logging level.
        Acceptable values: NOTSET, DEBUG, WARN, INFO, ERROR, CRITICAL
    """
    logger = logging.getLogger("pygameday")
    logger.setLevel(logging.getLevelName(level))


def validate_date(date):
    if type(date) == datetime:
        return date

    elif type(date) == str:
        try:
            date = parser.parse(date)

        except ValueError:
            msg = "Invalid date: {}. Aborting.".format(date)
            logger.critical(msg)
            sys.exit(1)

        return date

    else:
        msg = "Invalid date: {}. Aborting.".format(date)
        logger.critical(msg)
        sys.exit(1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Defines classes for database mappings, plus some database helper functions
"""
from __future__ import print_function, division, absolute_import

from sqlalchemy import Column
from sqlalchemy import Integer
from sqlalchemy import Float
from sqlalchemy import String
from sqlalchemy import Sequence
from sqlalchemy import DateTime
from sqlalchemy import ForeignKey
from sqlalchemy import create_engine
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base


BASE = declarative_base()


def db_connect(database_uri):
    """Connects to a database using database settings in config.py

    Parameters
    ----------
    database_uri : str
        The database URI.
        Examples: "postgresql+psycopg2://user:passwd@localhost/dbname"
                  "sqlite:///dbname.db"

    Returns
    -------
    engine : sqlalchemy engine instance
    """
    return create_engine(database_uri)


def create_db_tables(engine):
    """Creates database tables

    Parameters
    ----------
    engine : sqlalchemy engine instance
    """
    BASE.metadata.create_all(engine, checkfirst=True)


class Game(BASE):
    __tablename__ = 'games'

    game_id = Column(Integer, Sequence('game_id_seq'), primary_key=True)
    gameday_id = Column(String, unique=True, index=True)
    venue = Column(String)
    start_time = Column(DateTime)
    game_data_directory = Column(String)
    game_type = Column(String)
    home_name_abbrev = Column(String(3))
    home_team_city = Column(String)
    home_team_name = Column(String)
    away_name_abbrev = Column(String(3))
    away_team_city = Column(String)
    away_team_name = Column(String)
    home_team_runs = Column(Integer)
    away_team_runs = Column(Integer)
    league = Column(String)

    at_bats = relationship('AtBat', order_by='AtBat.at_bat_id', backref='games')
    hits_in_play = relationship('HitInPlay', order_by='HitInPlay.hip_id', backref='games')

    def __repr__(self):
        return "<Game(game_id=\'{}\', time=\'{}\', {} {} at {} {})>" \
                    .format(self.game_id, self.start_time,
                            self.away_team_city, self.away_team_name,
                            self.home_team_city, self.home_team_name)


class AtBat(BASE):
    __tablename__ = 'at_bats'

    at_bat_id = Column(Integer, Sequence('at_bat_id_seq'), primary_key=True)
    game_id = Column(Integer, ForeignKey('games.game_id'))
    inning = Column(Integer)
    inning_half = Column(String)
    n_pitches = Column(Integer)
    n_balls = Column(Integer)
    n_strikes = Column(Integer)
    n_outs = Column(Integer)
    batter_id = Column(Integer)
    pitcher_id = Column(Integer)
    batter_stance = Column(String)
    des = Column(String)
    event = Column(String)

    pitches = relationship('Pitch', order_by='Pitch.pitch_id', backref='at_bats')

    def __repr__(self):
        return "<AtBat(at_bat_id={}, batter_id={}, pitcher_id={}, des={})>" \
                .format(self.at_bat_id, self.batter_id, self.pitcher_id, self.des)


class Pitch(BASE):
    __tablename__ = 'pitches'

    pitch_id = Column(Integer, Sequence('pitch_id_seq'), primary_key=True)
    at_bat_id = Column(Integer, ForeignKey('at_bats.at_bat_id'))
    at_bat_pitch_num = Column(Integer)
    inning = Column(Integer)
    inning_half = Column(String)
    des = Column(String)
    result_type = Column(String)
    gameday_sv_id = Column(String)
    x = Column(Float)
    y = Column(Float)
    start_speed = Column(Float)
    end_speed = Column(Float)
    sz_top = Column(Float)
    sz_bot = Column(Float)
    pfx_x = Column(Float)
    pfx_z = Column(Float)
    px = Column(Float)
    pz = Column(Float)
    x0 = Column(Float)
    y0 = Column(Float)
    z0 = Column(Float)
    vx0 = Column(Float)
    vy0 = Column(Float)
    vz0 = Column(Float)
    ax = Column(Float)
    ay = Column(Float)
    az = Column(Float)
    break_y = Column(Float)
    break_angle = Column(Float)
    break_length = Column(Float)
    pitch_type = Column(String)
    type_conf = Column(String)
    zone = Column(Integer)
    nasty = Column(Integer)
    spin_dir = Column(Float)
    spin_rate = Column(Float)

    def __repr__(self):
        return "<Pitch(pitch_type={}, start_speed={}, result_type={}, des={})>" \
                .format(self.pitch_type, self.start_speed, self.result_type, self.des)


class Player(BASE):
    __tablename__ = 'players'

    player_id = Column(Integer, primary_key=True)
    first = Column(String)
    last = Column(String)
    boxname = Column(String)
    rl = Column(String)
    bats = Column(String)

    def __repr__(self):
        return "<Player(player_id={}, {} {})>" \
            .format(self.player_id, self.first, self.last)


class HitInPlay(BASE):
    __tablename__ = 'hits_in_play'

    hip_id = Column(Integer, Sequence('hip_id_sequence'), primary_key=True)
    game_id = Column(Integer, ForeignKey('games.game_id'))
    batter_id = Column(Integer)
    pitcher_id = Column(Integer)
    des = Column(String)
    hip_type = Column(String)
    team = Column(String)
    inning = Column(Integer)
    x = Column(Float)
    y = Column(Float)

    def __repr__(self):
        return "<HitInPlay(batter_id={}, pitcher_id={}, {})>" \
            .format(self.batter_id, self.pitcher_id, self.des)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Defines functionality for parsing MLB GameDay data from web content into database classes
"""
from __future__ import print_function, division, absolute_import

import logging

from dateutil import parser
from lxml import etree

from pygameday.models import AtBat
from pygameday.models import Game
from pygameday.models import HitInPlay
from pygameday.models import Pitch
from pygameday.models import Player

LOG = logging.getLogger("pygameday")


def parse_epg(epg_page):
    """Parse epg.xml to find all games in a given day

    Parameters
    ----------
    epg_page : requests page
        The result of a request for an epg.xml page

    Returns
    -------
    game_nodes : lxml list

    """
    root = etree.fromstring(epg_page.content)
    game_nodes = root.xpath('descendant::game')  # find all <game> nodes in the tree

    return game_nodes


def parse_game(game_node):
    """Parses a Game XML node into a Game database object

    Parameters
    ----------
    game_node : lxml node
        The Game node to parse

    Returns
    -------
    A Game object
    """
    db_game = None

    # Only parse games if they are final
    if game_node.get("status") in ["Final", "Completed Early"]:

        start_datetime = parser.parse(game_node.get("start"))

        db_game = Game(gameday_id=game_node.get("id"),
                       venue=game_node.get("venue"),
                       start_time=start_datetime,
                       game_data_directory=game_node.get("game_data_directory"),
                       game_type=game_node.get("game_type"),
                       home_name_abbrev=game_node.get("home_name_abbrev"),
                       home_team_city=game_node.get("home_team_city"),
                       home_team_name=game_node.get("home_team_name"),
                       away_name_abbrev=game_node.get("away_name_abbrev"),
                       away_team_city=game_node.get("away_team_city"),
                       away_team_name=game_node.get("away_team_name"),
                       home_team_runs=game_node.get("home_team_runs"),
                       away_team_runs=game_node.get("away_team_runs"),
                       league=game_node.get("league")
                       )
    else:
        msg = "GameDay ID {} was not parsed because its status is {}" \
                .format(game_node.get("id"), game_node.get("status"))
        LOG.info(msg)

    return db_game


def parse_players(players_page):
    """Parses a players.xml page

    Parameters
    ----------
    players_page : XML data
        The player page retrieved by the requests module

    Returns
    -------
    list
        A list of Player database objects
    """
    root = etree.fromstring(players_page.content)
    player_nodes = root.xpath('descendant::player')  # find all <player> nodes

    db_players_list = [parse_player_node(p) for p in player_nodes]
    return db_players_list


def parse_player_node(player_node):
    """Parses a player XML node

    Parameters
    ----------
    player_node : lxml node
        The player node to parse

    Returns
    -------
    A Player database object
    """
    db_player = Player(player_id=player_node.get("id"),
                       first=player_node.get("first"),
                       last=player_node.get("last"),
                       boxname=player_node.get("boxname"),
                       rl=player_node.get("rl"),
                       bats=player_node.get("bats"),
                       )
    return db_player


def parse_hit_chart(hit_chart_page):
    """Parses inning_hit.xml

    Parameters
    ----------
    hit_chart_page
        The dadta from inning_hit.xml
    """
    root = etree.fromstring(hit_chart_page.content)
    hip_nodes = root.xpath('descendant::hip')  # find all <hip> nodes

    db_hips_list = [parse_hit_in_play_node(h) for h in hip_nodes]
    return db_hips_list


def parse_hit_in_play_node(hip_node):
    """Parses a Hit In Play node

    Parameters
    ----------
    hip_node : lxml node
        The hit in play node to parse

    Returns
    -------
    A HitInPlay object
    """
    hip = HitInPlay(batter_id=hip_node.get("batter"),
                    pitcher_id=hip_node.get("pitcher"),
                    des=hip_node.get("des"),
                    hip_type=hip_node.get("type"),
                    team=hip_node.get("team"),
                    inning=hip_node.get("inning"),
                    x=hip_node.get("x"),
                    y=hip_node.get("y")
                    )
    return hip


def parse_inning_all(inning_all_page):
    """Parses inning_all.xml for atbats and pitches

    Parameters
    ----------
    inning_all_page
        The data from inning_all.xml
    """
    root = etree.fromstring(inning_all_page.content)
    inning_nodes = root.xpath('descendant::inning')  # Find all <atbat> nodes

    db_at_bat_list = []
    ab_nodes_top = []
    ab_nodes_bot = []

    for inn in inning_nodes:
        inning_num = inn.get("num")  # the inning number

        inning_top = inn.xpath('top')  # the top of the inning, as a list
        inning_bot = inn.xpath('bottom')  # the bottom of the inning, as a list

        if inning_top:  # if the top of the inning exists (is not empty)
            ab_nodes_top = inning_top[0].xpath('descendant::atbat')

        if inning_bot:  # if the bottom of the inning exists (is not empty)
            ab_nodes_bot = inning_bot[0].xpath('descendant::atbat')

        for ab in ab_nodes_top:
            db_at_bat = parse_at_bat(ab, inning_num, 'T')
            db_at_bat_list.append(db_at_bat)

        for ab in ab_nodes_bot:
            db_at_bat = parse_at_bat(ab, inning_num, 'B')
            db_at_bat_list.append(db_at_bat)

    return db_at_bat_list


def parse_at_bat(at_bat, inning_num, inning_half):
    """Parses an at bat XML node

    Parameters
    ----------
    at_bat : lxml node
        The at bat node to parse
    inning_num : int
        The inning number (e.g., 1 for first inning)
    inning_half : str
        'T' if top of the inning, 'B' if bottom of the inning

    Returns
    -------
    An AtBat database object
    """
    pitches = at_bat.xpath('descendant::pitch')  # find all <pitch> nodes

    db_at_bat = AtBat(
            inning=inning_num,
            inning_half=inning_half,
            n_pitches=len(pitches),
            n_balls=at_bat.get("b"),
            n_strikes=at_bat.get("s"),
            n_outs=at_bat.get("o"),
            batter_id=at_bat.get("batter"),
            pitcher_id=at_bat.get("pitcher"),
            batter_stance=at_bat.get("stand"),
            des=at_bat.get("des"),
            event=at_bat.get("event")
            )

    for index, pitch in enumerate(pitches):
        db_pitch = parse_pitch(pitch, inning_num, inning_half, index)
        db_at_bat.pitches.append(db_pitch)

    return db_at_bat


def parse_pitch(pitch, inning_num, inning_half, index):
    """Parses a pitch XML node

    Parameters
    ----------
    pitch : lxml node
        The pitch node to parse
    inning_num : int
        The inning number (e.g., 1 for first inning)
    inning_half : str
        'T' if top of the inning, 'B' if bottom of the inning
    index : int
        Index of the pitch within the ab-bat it occurred in

    Returns
    -------
    A Pitch database object
    """
    db_pitch = Pitch(
            at_bat_pitch_num=index,
            inning=inning_num,
            inning_half=inning_half,
            des=pitch.get("des"),
            result_type=pitch.get("type"),
            gameday_sv_id=pitch.get("sv_id"),
            x=pitch.get("x"),
            y=pitch.get("y"),
            start_speed=pitch.get("start_speed"),
            end_speed=pitch.get("end_speed"),
            sz_top=pitch.get("sz_top"),
            sz_bot=pitch.get("sz_bot"),
            pfx_x=pitch.get("pfx_x"),
            pfx_z=pitch.get("pfx_z"),
            px=pitch.get("px"),
            pz=pitch.get("pz"),
            x0=pitch.get("x0"),
            y0=pitch.get("y0"),
            z0=pitch.get("z0"),
            vx0=pitch.get("vx0"),
            vy0=pitch.get("vy0"),
            vz0=pitch.get("vz0"),
            ax=pitch.get("ax"),
            ay=pitch.get("ay"),
            az=pitch.get("az"),
            break_y=pitch.get("break_y"),
            break_angle=pitch.get("break_angle"),
            break_length=pitch.get("break_length"),
            pitch_type=pitch.get("pitch_type"),
            type_conf=pitch.get("type_conf"),
            zone=pitch.get("zone"),
            nasty=pitch.get("nasty"),
            spin_dir=pitch.get("spin_dir"),
            spin_rate=pitch.get("spin_rate")
            )
    return db_pitch


def save_page(page, filename):
    """Writes the text of an html page to disk

    Parameters
    ----------
    page : requests page object
        The page to write
    filename
        The name of the file to create
    """
    with open(filename, 'w') as f:
        f.write(page.text)


def test():
    game_directory = '/components/game/mlb/year_2014/month_04/day_04/gid_2014_04_04_anamlb_houmlb_1'

#    print "Fetching epg page"
#    epg_page = gds.fetch_epg(year, month, day)
#    print "Parsing epg page"
#    parser.parse_epg(epg_page)
#
#    print "Fetching players page"
#    players_page = gds.fetch_players(game_directory)
#    print "Parsing players page"
#    parser.parse_players(players_page)
#
#    print "Fetching hit chart page"
#    hit_chart_page = gds.fetch_hit_chart(game_directory)
#    print "Parsing hit chart page"
#    parser.parse_hit_chart(hit_chart_page)
    from pygameday import scrape

    print("Fetching innings_all page")
    inning_all_page = scrape.fetch_inning_all(game_directory)
    print("Parsing innings_all page")
    at_bats = parse_inning_all(inning_all_page)
    print(at_bats)


if __name__ == "__main__":
    test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, division, absolute_import

from pygameday import util
from pygameday.pygameday import GameDayClient

__all__ = ["GameDayClient"]

# Set up logging
util.configure_logging("pygameday")




from distutils.core import setup
setup(
  name = 'pygameday',
  packages = ['pygameday'],
  version = '0.2',
  description = 'Module for scraping, parsing, and ingesting MLB GameDay data into a database',
  author = 'Chris Anderson',
  author_email = 'christian.c.anderson@gmail.com',
  url = 'https://github.com/raveneyes/pygameday',
  download_url = 'https://github.com/raveneyes/pygameday/tarball/0.1',
  keywords = ['baseball', 'gameday', 'database', 'scraping'],
  classifiers = [],
)
